#include "FWCore/MessageLogger/interface/MessageLogger.h"

using namespace std;

namespace {
  string onlyLowestDirectory( const string & file)
  {
    string::size_type lastSlash = file.find_last_of('/');
    if (lastSlash == string::npos) return file;
    if (lastSlash == file.size()-1)     return file;
    return file.substr(lastSlash+1, file.size()-lastSlash-1);
  }

  edm::Level myLevel=edm::Warning;
}

void edm::setLogLevel ( Level l )
{
  myLevel=l;
}

edm::LogError::LogError( string const & id ) : LogBase ( Error )
{
  if ( write() )
    cout << raveimpl::colors::red() << "[" << id << "] "; 
}

edm::LogWarning::LogWarning( string const & id ): LogBase ( Warning )
{
  if ( write() )
    cout << raveimpl::colors::yellow() << "[" << id << "] "; 
}

edm::LogInfo::LogInfo( string const & id ) : LogBase ( Info )
{
  if ( write() )
    cout << raveimpl::colors::green() << "[" << id << "] "; 
}

edm::LogDebug_::LogDebug_( string const & id, string const & file, int line ) : LogBase ( Debug )
{
  if ( write() )
  {
    cout << raveimpl::colors::cyan() << "[";
    if ( id.size() ) cout << id << " ";
    cout << onlyLowestDirectory( file ) << ":" << line << "] "; 
  }
}

edm::LogDebug_::LogDebug_( string const & id ) : LogBase ( Debug )
{
  if ( write() )
    cout << raveimpl::colors::cyan() << "[" << id << "] "; 
}

edm::LogBase::LogBase ( Level l ) : level_(l) {}

edm::LogBase::~LogBase()
{
  if ( write() )
  {
    cout << raveimpl::colors::reset() << endl;
  }
}

edm::LogBase & edm::LogBase::operator<< ( ostream&(*f)(ostream&))  
{
  if ( write() )
  {
    cout << raveimpl::colors::reset() << endl;
  }
  return *this;
}

bool edm::LogBase::write() const
{
  return ( myLevel >= level_ );
}

edm::LogBase & edm::LogBase::operator<< ( ios_base&(*f)(ios_base&) )  
{
  if ( write() )
  {
    cout << raveimpl::colors::reset() << endl;
  }
  return *this;
}     
