#include "RaveBase/Converters/interface/CmsToRaveObjects.h"
#include "RaveBase/RaveInterface/rave/RaveVector6D.h"
#include "RaveBase/RaveInterface/rave/RaveCovariance6D.h"
#include "TrackingTools/TrajectoryParametrization/interface/CartesianTrajectoryError.h"
#include "RaveBase/Converters/interface/RaveStreamers.h"
#include <limits>

using namespace std;
namespace {
    rave::Track getNr ( const vector < rave::Track > & trks, int id )
    {
        for ( vector< rave::Track >::const_iterator i=trks.begin(); 
              i!=trks.end() ; ++i )
        {
            if ( i->id() == id ) return *i;
        }
        cout << "[CmsToRaveObjects] no track nr " << id << "?!?" << endl;
        exit(0);
    }

    typedef numeric_limits<float> num;
}


rave::Track CmsToRaveObjects::convert ( const TrajectoryStateOnSurface & s ) const
{
  rave::Vector6D st=convert ( s.globalParameters() );
  rave::Covariance6D err = convert ( s.cartesianError() );
  return rave::Track ( st, err, s.charge(), num::quiet_NaN() /*chi2*/, 
                       num::quiet_NaN ()/*ndof*/, 0, "" );
}

rave::Track CmsToRaveObjects::convert ( const FreeTrajectoryState & s ) const
{
  rave::Vector6D st=convert ( s.parameters() );
  rave::Covariance6D err = convert ( s.cartesianError() );
  return rave::Track ( st, err, s.charge(), num::quiet_NaN() /*chi2*/,                                          
                                 num::quiet_NaN ()/*ndof*/, 0, "" );
}

rave::Vector6D CmsToRaveObjects::convert ( const GlobalTrajectoryParameters & p ) const
{
  return rave::Vector6D ( p.position().x(), p.position().y(), p.position().z(),
                          p.momentum().x(), p.momentum().y(), p.momentum().z() );

}

rave::Covariance6D CmsToRaveObjects::convert ( const CartesianTrajectoryError & e ) const
{
  return rave::Covariance6D ( e.matrix()[0][0], e.matrix()[0][1], e.matrix()[0][2],
                              e.matrix()[1][1], e.matrix()[1][2], e.matrix()[2][2],
                              e.matrix()[0][3], e.matrix()[0][4], e.matrix()[0][5],
                              e.matrix()[1][3], e.matrix()[1][4], e.matrix()[1][5],
                              e.matrix()[2][3], e.matrix()[2][4], e.matrix()[2][5],
                              e.matrix()[3][3], e.matrix()[3][4], e.matrix()[3][5],
                              e.matrix()[4][4], e.matrix()[4][5], e.matrix()[5][5] );
}

rave::Covariance3D CmsToRaveObjects::convert ( const GlobalError & e ) const
{
    return rave::Covariance3D( e.cxx(), e.cyx(), e.czx(),
                               e.cyy(), e.czy(), e.czz() );
}

rave::Vertex CmsToRaveObjects::convert( const TransientVertex & tv,
       const vector < rave::Track > & trks ) const
{
    vector < pair < double, rave::Track > > tracks;

    map<reco::TransientTrack,float > wm=tv.weightMap();
    
    for ( map<reco::TransientTrack,float >::iterator iter = wm.begin();
          iter != wm.end();
          iter++ )
    {
        tracks.push_back( pair< double, rave::Track >( iter->second, getNr ( trks, iter->first.id() ) ) );
    }

    return rave::Vertex( rave::Point3D( tv.position().x(),
                                        tv.position().y(),
                                        tv.position().z() ),
                   CmsToRaveObjects().convert( tv.positionError() ),
                   tracks,
                   static_cast<double>( tv.degreesOfFreedom() ),
                   static_cast<double>( tv.totalChiSquared() ) );
}
