#include "RaveBase/Converters/interface/RaveStreamers.h"

using namespace std;

namespace {
  bool isDiagonal ( const rave::Covariance3D & cov )
  {
    if ( fabs ( cov.dxy() ) > 1e-7 ) return false;
    if ( fabs ( cov.dxz() ) > 1e-7 ) return false;
    if ( fabs ( cov.dyz() ) > 1e-7 ) return false;
    return true;
  }
}

ostream& operator<< (ostream& os, const rave::Point3D & p )
{
  os << "(" << p.x() << ", " << p.y() << ", " << p.z() << ")";
  return os;
}

ostream& operator<< (ostream& os, const rave::Covariance3D & p )
{
  os << "(" << p.dxx() << endl 
     << " " << p.dxy() << " " << p.dyy() << endl
     << " " << p.dxz() << " " << p.dyz() << " " << p.dzz() << ")" << endl;
  return os;
}


ostream& operator<< (ostream& os, const rave::Ellipsoid3D & p )
{
  rave::Point3D pt=p.point();
  rave::Covariance3D cov=p.covariance();
  if ( isDiagonal ( cov ) )
  {
    os << "(" << pt.x() << "+/-" << sqrt ( cov.dxx() )
       << ", " << pt.y() << "+/-" << sqrt ( cov.dyy() )
       << ", " << pt.z() << "+/-" << sqrt ( cov.dzz() )
       << ")";
  } else {
    os << pt << " cov=" << cov;
  }
  return os;
}


/*
ostream& operator<< (ostream& os, const rave::Vector3D & p )
{
  os << "(" << p.x() << ", " << p.y() << ", " << p.z() << ")";
  return os;
}*/

ostream& operator<< (ostream& os, const rave::Vector6D & p )
{
  os << "(" << p.x() << ", " << p.y() << ", " << p.z() << " , " 
     << p.px() << ", " << p.py() << ", " << p.pz() << ")";
  return os;
}

ostream& operator<< (ostream& os, const rave::Track & t )
{
  os << t.state();
  return os;
}

ostream& operator<< (ostream& os, const rave::Vertex & v )
{
  os << "id: " << v.id() << ", position: " << v.position();
  return os;
}

