#include "RaveBase/Converters/interface/RaveToCmsObjects.h"
#include "RaveBase/RaveInterface/rave/RaveVector6D.h"
#include "DataFormats/GeometryVector/interface/GlobalPoint.h"
#include "DataFormats/GeometryVector/interface/GlobalVector.h"
#include "RaveBase/Converters/interface/MagneticFieldWrapper.h"
#include "TrackingTools/TransientTrack/interface/TransientTrack.h"
#include <iostream>
#include <vector>

using namespace std;

GlobalTrajectoryParameters RaveToCmsObjects::convert ( const rave::Track & s ) const
{
  GlobalPoint pos ( s.state().x(), s.state().y(), s.state().z() );
  GlobalVector mom ( s.state().px(), s.state().py(), s.state().pz() );
  return GlobalTrajectoryParameters ( pos , mom, s.charge(), MagneticFieldWrapper::Instance() ); 
}

CartesianTrajectoryError RaveToCmsObjects::convert ( const rave::Covariance6D & c ) const
{
  AlgebraicSymMatrix s(6,0);
  s[0][0]=c.dxx();
  s[0][1]=c.dxy();
  s[0][2]=c.dxz();
  s[0][3]=c.dxpx();
  s[0][4]=c.dxpy();
  s[0][5]=c.dxpz();
  s[1][1]=c.dyy();
  s[1][2]=c.dyz();
  s[1][3]=c.dypx();
  s[1][4]=c.dypy();
  s[1][5]=c.dypz();
  s[2][2]=c.dzz();
  s[2][3]=c.dzpx();
  s[2][4]=c.dzpy();
  s[2][5]=c.dzpz();
  s[3][3]=c.dpxpx();
  s[3][4]=c.dpxpy();
  s[3][5]=c.dpxpz();
  s[4][4]=c.dpypy();
  s[4][5]=c.dpypz();
  s[5][5]=c.dpzpz();
  return CartesianTrajectoryError ( s );
}


GlobalPoint RaveToCmsObjects::convert ( const rave::Vector3D & v ) const
{
  return GlobalPoint ( v.x(), v.y(), v.z() );
}

GlobalError RaveToCmsObjects::convert ( const rave::Covariance3D & c ) const
{
  return GlobalError ( c.dxx(), c.dxy(), c.dyy(), c.dxz(), c.dyz(), c.dzz() );
}

TransientVertex RaveToCmsObjects::convert ( const rave::Vertex & v ) const
{
  GlobalPoint pos = convert ( v.position() );
  GlobalError err ( convert ( v.error() ) );
  vector < reco::TransientTrack > trks;
  vector< rave::Track > orig=v.tracks();
  for ( vector< rave::Track >::const_iterator i=orig.begin(); i!=orig.end() ; ++i )
  {
    trks.push_back ( reco::TransientTrack ( *i ) );
  }
  return TransientVertex ( pos, err, trks, v.chiSquared() );
}
