#include "RaveBase/RaveInterface/rave/BasicRaveTrack.h"
#include "RaveBase/RaveEngine/interface/RaveId.h"
#include <iostream>

using namespace rave;

BasicTrack::BasicTrack( const Vector6D & s, const Covariance6D & e, Charge q,
   double chi2, double ndof, void * p, std::string d )
  : theState ( s ), theError ( e ), theCharge ( q ), thePointer ( p ),
    theTag ( d ), theId ( RaveId::uniqueId() ), theIsValid ( true ),
    theChi2( chi2 ), theNdof ( ndof )
{
  // theComponents.push_back ( *this );
}

double BasicTrack::chi2() const
{
  return theChi2;
}

double BasicTrack::ndof() const
{
  return theNdof;
}

std::string BasicTrack::tag() const
{
  return theTag;
}


bool BasicTrack::isValid() const
{
  return theIsValid;
}

Charge BasicTrack::charge() const
{
  return theCharge;
}

int BasicTrack::id() const
{
  return theId;
}

const Vector6D & BasicTrack::state() const
{
  return theState;
}

const Covariance6D & BasicTrack::error() const
{
  return theError;
}

void * BasicTrack::originalObject() const
{
  return thePointer;
}

BasicTrack * BasicTrack::clone() const
{
  return new BasicTrack ( *this );
}

BasicTrack::BasicTrack ( const std::vector < BasicTrack > & components ) :
  theIsValid ( true ), theComponents ( components )
{
  // FIXME compute weighted mean
  std::cout << "[BasicTrack] error, missing implementation for multi-component track."
            << std::endl;
}

BasicTrack::BasicTrack () : theId(-1), theIsValid ( false )
{}

std::vector < BasicTrack > BasicTrack::components() const
{
  return theComponents;
}

bool BasicTrack::operator< ( const BasicTrack & o ) const
{
    return ( id() < o.id() );
}
