#include "RaveBase/RaveInterface/rave/RaveVertex.h"
#include "RaveBase/RaveEngine/interface/RaveId.h"

using namespace rave;
using namespace std;

Vertex::Vertex( const Point3D & p, const Covariance3D & e,
      const vector < pair < double, Track > > & tracks, double ndf, double chi2 ) :
  Base ( new BasicVertex ( p, e, tracks, ndf, chi2 ) )
{}

Vertex::Vertex( const Point3D & p, const Covariance3D & e,
      const vector < Track > & tracks, double ndf, double chi2 ) :
  Base ( new BasicVertex ( p, e, tracks, ndf, chi2 ) )
{
}

const Point3D & Vertex::position() const
{
  return data().position();
}

const Covariance3D & Vertex::error() const
{
  return data().error();
}

int Vertex::id() const
{
  return data().id();
}


const vector < pair < double, Track > > & Vertex::weightedTracks() const
{
  return data().weightedTracks();
}

vector < Track > Vertex::tracks() const
{
  return data().tracks();
}

bool Vertex::operator< ( const Vertex & o ) const
{
    return ( id() < o.id() );
}

bool Vertex::operator== ( const Vertex & o ) const
{
  return ( id() == o.id() );
}

double Vertex::ndf() const
{
  return data().ndf();
}

double Vertex::chiSquared() const
{
  return data().chiSquared();
}

bool Vertex::isValid() const
{
  return data().isValid();
}

Vertex::Vertex () : Base ( new BasicVertex() )
{}

Vertex::Vertex ( const BasicVertex & o ) : Base ( new BasicVertex ( o ) )
{}

std::vector < Vertex > Vertex::components() const
{
  std::vector < Vertex > ret;
  for ( std::vector< BasicVertex >::const_iterator i=data().components().begin();
        i!=data().components().end() ; ++i )
  {
    ret.push_back ( Vertex ( *i ) );
  }
  return ret;
}
