#include "RecoBTag/RaveTools/interface/AlgorithmToRaveConverter.h"
#include "FWCore/MessageLogger/interface/MessageLogger.h"

using namespace std;

void AlgorithmToRaveConverter::add ( rave::Calibration & cal, const vector < pair < 
                        combsv::CombinedSVCalibrationCategory *, 
                        combsv::CombinedSVCalibrationHistogram * > > & histos ) const
{
  for ( vector< pair < combsv::CombinedSVCalibrationCategory *, 
                       combsv::CombinedSVCalibrationHistogram * > >::const_iterator 
        i=histos.begin(); i!=histos.end() ; ++i )
  {
    add ( cal, *(i->first), *(i->second) );
  }
}

void AlgorithmToRaveConverter::add ( rave::Calibration & cal,
     const combsv::CombinedSVCalibrationCategory & cat,
     const combsv::CombinedSVCalibrationHistogram & histo,
     bool init ) const
{
  rave::CalibrationHistogram h ( cat.taggingVar(), cat.jetPtMin(), cat.jetPtMax(), cat.jetEtaMin(),
                               cat.jetEtaMax(), cat.vertexType(), cat.partonType() );
  const vector < float > & bins = histo.upperLimits();
  for ( unsigned i=0; i< bins.size() ; i++ )
  {
    // h.add ( bins[i], 23. );
    // double bc=histo.binContent(i);
    /*
    if ( bc > 0.1 )
    {
    }*/
    // FIXME crashes if no .binContent ( i )
    if ( init )
    {
      h.add ( bins[i], 0., 0 );
    } else {
      h.add ( bins[i], histo.binContent ( i ), 0 );
    }
  }
  h.addEntries ( histo.entries() );
  cal.add(h);  
}

rave::Calibration AlgorithmToRaveConverter::convert ( 
    const vector < pair < combsv::CombinedSVCalibrationCategory *, 
                          combsv::CombinedSVCalibrationHistogram * > > & histos ) const
{
  rave::Calibration ret;
  add ( ret, histos );
  return ret;
}

rave::Calibration AlgorithmToRaveConverter::convert ( 
    const BTagAlgorithmCalibration * a ) const
{
  return convert ( a->dumpCalibration() );
}

rave::Calibration AlgorithmToRaveConverter::convert ( const BTagAlgorithmCalibration * reco,
                            const BTagAlgorithmCalibration * pseudo,
                            const BTagAlgorithmCalibration * no ) const
{
  rave::Calibration ret;
  add ( ret, reco->dumpCalibration() );
  add ( ret, pseudo->dumpCalibration() );
  add ( ret, no->dumpCalibration() );
  return ret;
}
