#include "RecoBTag/RaveTools/interface/RaveAlgorithmConverter.h"
#include "DataFormats/BTauReco/interface/TaggingVariable.h"
#include "DataFormats/BTauReco/interface/VertexTypes.h"
#include "RecoBTag/CombinedSV/interface/PartonTypes.h"
#include "FWCore/MessageLogger/interface/MessageLogger.h"

using namespace std;

namespace {
  int debug() { return 0; }
}

vector < pair < combsv::CombinedSVCalibrationCategory *, combsv::CombinedSVCalibrationHistogram * > >
  RaveAlgorithmConverter::convert ( const rave::Calibration & c ) const
{
  vector < pair < combsv::CombinedSVCalibrationCategory *, combsv::CombinedSVCalibrationHistogram * > > ret;
  for ( vector< rave::CalibrationHistogram >::const_iterator i=c.begin(); i!=c.end() ; ++i )
  {
    // if ( i->entries() > 0 ) edm::LogError ( "RaveAlgorithmConverter") << "entries " << i->entries(); 
    combsv::CombinedSVCalibrationHistogram * histo = 
      new combsv::CombinedSVCalibrationHistogram( i->jetPtMin(),
        i->jetPtMax(), i->jetEtaMin(), i->jetEtaMax(), 
        reco::btag::Vertices::VertexType ( i->vertexType() ),
        combsv::Partons::PartonType ( i->partonType() ),
        reco::btau::TaggingVariableName ( i->taggingVariable() ),
        i->getULimits(), i->entries() );
    map < float, float > bins = i->getMap();
    int ctr=0;
    for ( map < float, float >::const_iterator i=bins.begin(); i!=bins.end() ; ++i )
    {
      histo->setBinContent ( ctr, i->second );
      ctr++;
    }

    ret.push_back(pair
        < combsv::CombinedSVCalibrationCategory *, combsv::CombinedSVCalibrationHistogram * > ( 
          &(histo->category()), histo ) );

    // if ( histo->entries() > 0 ) 
    // edm::LogError ( "RaveAlgorithmConverter" ) << histo->entries() << " entries.";

    if ( debug() )
    {
      edm::LogInfo ( "RaveAlgorithmConverter") << "creating category " << histo->category() << endl;

      histo->dump();

    }
  }

  return ret;
}

/*
BTagAlgorithmCalibration RaveAlgorithmConverter::create ( const rave::Calibration & c ) const
{
  return BTagAlgorithmCalibration ( convert ( c ) );
}*/
